@@ -100,16 +100,22 @@ getInnerSegment() {
             {"kuan", {"ku", "an"}},     {"kuai", {"ku", "ai"}},
             {"nuan", {"nu", "an"}},     {"piao", {"pi", "ao"}},
             {"pian", {"pi", "an"}},     {"quan", {"qu", "an"}},
-            {"juan", {"ju", "an"}},     {"qiao", {"qi", "ao"}},
+            {"quang", {"qu", "ang"}},   {"qvan", {"qv", "an"}},
+            {"qvang", {"qv", "ang"}},   {"juan", {"ju", "an"}},
+            {"juang", {"ju", "ang"}},   {"jvan", {"jv", "an"}},
+            {"jvang", {"jv", "ang"}},   {"qiao", {"qi", "ao"}},
             {"qiang", {"qi", "ang"}},   {"qian", {"qi", "an"}},
-            {"yuan", {"yu", "an"}},     {"zhuang", {"zhu", "ang"}},
-            {"zhuan", {"zhu", "an"}},   {"zhuai", {"zhu", "ai"}},
-            {"niao", {"ni", "ao"}},     {"niang", {"ni", "ang"}},
-            {"nian", {"ni", "an"}},     {"liao", {"li", "ao"}},
-            {"liang", {"li", "ang"}},   {"lian", {"li", "an"}},
-            {"zuan", {"zu", "an"}},     {"tuan", {"tu", "an"}},
-            {"tiao", {"ti", "ao"}},     {"tian", {"ti", "an"}},
-            {"xuan", {"xu", "an"}},     {"suan", {"su", "an"}},
+            {"yuang", {"yu", "ang"}},   {"yvang", {"yv", "ang"}},
+            {"yuan", {"yu", "an"}},     {"yvan", {"yv", "an"}},
+            {"zhuang", {"zhu", "ang"}}, {"zhuan", {"zhu", "an"}},
+            {"zhuai", {"zhu", "ai"}},   {"niao", {"ni", "ao"}},
+            {"niang", {"ni", "ang"}},   {"nian", {"ni", "an"}},
+            {"liao", {"li", "ao"}},     {"liang", {"li", "ang"}},
+            {"lian", {"li", "an"}},     {"zuan", {"zu", "an"}},
+            {"tuan", {"tu", "an"}},     {"tiao", {"ti", "ao"}},
+            {"tian", {"ti", "an"}},     {"xuang", {"xu", "ang"}},
+            {"xvang", {"xv", "ang"}},   {"xuan", {"xu", "an"}},
+            {"xvan", {"xv", "an"}},     {"suan", {"su", "an"}},
             {"biao", {"bi", "ao"}},     {"bian", {"bi", "an"}},
             {"shuang", {"shu", "ang"}}, {"shuan", {"shu", "an"}},
             {"shuai", {"shu", "ai"}},   {"ruan", {"ru", "an"}},
@@ -200,7 +206,17 @@ const PinyinMap &getPinyinMap() {
          {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"yuang", PinyinInitial::Y, PinyinFinal::UAN,
          PinyinFuzzyFlag::UAN_UANG},
+        {"yvagn",
+         PinyinInitial::Y,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
+        {"yvang",
+         PinyinInitial::Y,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"yuan", PinyinInitial::Y, PinyinFinal::UAN, PinyinFuzzyFlag::None},
+        {"yvan", PinyinInitial::Y, PinyinFinal::UAN,
+         PinyinFuzzyFlag::CommonTypo},
         {"yu", PinyinInitial::Y, PinyinFinal::U, PinyinFuzzyFlag::None},
         {"yv", PinyinInitial::Y, PinyinFinal::U, PinyinFuzzyFlag::CommonTypo},
         {"you", PinyinInitial::Y, PinyinFinal::OU, PinyinFuzzyFlag::None},
@@ -229,7 +245,17 @@ const PinyinMap &getPinyinMap() {
          {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"xuang", PinyinInitial::X, PinyinFinal::UAN,
          PinyinFuzzyFlag::UAN_UANG},
+        {"xvagn",
+         PinyinInitial::X,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
+        {"xvang",
+         PinyinInitial::X,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"xuan", PinyinInitial::X, PinyinFinal::UAN, PinyinFuzzyFlag::None},
+        {"xvan", PinyinInitial::X, PinyinFinal::UAN,
+         PinyinFuzzyFlag::CommonTypo},
         {"xu", PinyinInitial::X, PinyinFinal::U, PinyinFuzzyFlag::None},
         {"xv", PinyinInitial::X, PinyinFinal::U, PinyinFuzzyFlag::CommonTypo},
         {"xou", PinyinInitial::X, PinyinFinal::U, PinyinFuzzyFlag::U_OU},
@@ -403,7 +429,17 @@ const PinyinMap &getPinyinMap() {
          {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"quang", PinyinInitial::Q, PinyinFinal::UAN,
          PinyinFuzzyFlag::UAN_UANG},
+        {"qvagn",
+         PinyinInitial::Q,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
+        {"qvang",
+         PinyinInitial::Q,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"quan", PinyinInitial::Q, PinyinFinal::UAN, PinyinFuzzyFlag::None},
+        {"qvan", PinyinInitial::Q, PinyinFinal::UAN,
+         PinyinFuzzyFlag::CommonTypo},
         {"qu", PinyinInitial::Q, PinyinFinal::U, PinyinFuzzyFlag::None},
         {"qv", PinyinInitial::Q, PinyinFinal::U, PinyinFuzzyFlag::CommonTypo},
         {"qiu", PinyinInitial::Q, PinyinFinal::IU, PinyinFuzzyFlag::None},
@@ -612,7 +648,17 @@ const PinyinMap &getPinyinMap() {
          {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"juang", PinyinInitial::J, PinyinFinal::UAN,
          PinyinFuzzyFlag::UAN_UANG},
+        {"jvagn",
+         PinyinInitial::J,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
+        {"jvang",
+         PinyinInitial::J,
+         PinyinFinal::UAN,
+         {PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::CommonTypo}},
         {"juan", PinyinInitial::J, PinyinFinal::UAN, PinyinFuzzyFlag::None},
+        {"jvan", PinyinInitial::J, PinyinFinal::UAN,
+         PinyinFuzzyFlag::CommonTypo},
         {"ju", PinyinInitial::J, PinyinFinal::U, PinyinFuzzyFlag::None},
         {"jv", PinyinInitial::J, PinyinFinal::U, PinyinFuzzyFlag::CommonTypo},
         {"jiu", PinyinInitial::J, PinyinFinal::IU, PinyinFuzzyFlag::None},
