@@ -28,6 +28,12 @@ static std::string applyFuzzy(const std::string &str, PinyinFuzzyFlags flags) {
         if (boost::algorithm::ends_with(result, "ve")) {
             result[result.size() - 2] = 'u';
         }
+        if (boost::algorithm::ends_with(result, "van")) {
+            result[result.size() - 3] = 'u';
+        }
+        if (boost::algorithm::ends_with(result, "vang")) {
+            result[result.size() - 4] = 'u';
+        }
     }
 
     if (flags & PinyinFuzzyFlag::VE_UE) {
